searchNodes=[{"doc":"Vela is a tiny library providing easy management of validated cached state with some history.Including use Vela in your module would turn the module into struct, setting field accordingly to the specification, passed as a parameter.Vela allows the following configurable parameters per field:limit — length of the series to keep (default: 5)compare_by — comparator extraction function to extract the value, to be used for comparison, from the underlying terms (default: &amp; &amp;1)comparator — the function that accepts a series name and two values and returns the greater one to be used in Vela.δ/1 (default: &amp;&lt;/2)threshold — if specified, the inserted value is checked to fit in δ ± threshold; whether it does not, it goes to errors (float() | nil, default: nil)validator — the function to be used to invalidate the accumulated values (default: fn _ -&gt; true end)sorter — the function to be used to sort values within one serie, if none is given, it sorts in the natural order, FIFO, newest is the one to poperrors — number of errors to keep (default: 5)Also, Vela accepts :mη keyword parameter for the cases when the consumer needs the very custom meta to be passed to the struct.Vela implements Access behaviour.Usagedefmodule Vela.Test do use Vela, series1: [limit: 3, errors: 1], # no validation series2: [limit: 2, validator: Vela.Test] series3: [ compare_by: &amp;Vela.Test.comparator/1, validator: &amp;Vela.Test.validator/2 ] @behaviour Vela.Validator @impl Vela.Validator def valid?(_serie, value), do: value &gt; 0 @spec comparator(%{created_at :: DateTime.t()}) :: DateTime.t() def comparator(%{created_at: created_at}), do: created_at @spec validator(value :: t()) :: boolean() def validator(value), do: is_integer(value) and value &gt; 300 endIn the example above, before any structure update attempt (via Access,) this valid?/2 function would be called.If it returns true, the value gets inserted / updated, and the series behind is truncated if needed. It it returns false, the state is not updated, and the value is put into the map under __errors__ key of the struct. The length of errors is also configurable via errors: keyword parameter.","ref":"Vela.html","title":"Vela","type":"module"},{"doc":"","ref":"Vela.html#flat_map/2","title":"Vela.flat_map/2","type":"function"},{"doc":"","ref":"Vela.html#map/2","title":"Vela.map/2","type":"function"},{"doc":"","ref":"Vela.html#validator!/2","title":"Vela.validator!/2","type":"function"},{"doc":"Represents a key-value pair in errors and unmatched","ref":"Vela.html#t:kv/0","title":"Vela.kv/0","type":"type"},{"doc":"Represents a key in the Vela structure","ref":"Vela.html#t:serie/0","title":"Vela.serie/0","type":"type"},{"doc":"Represents the struct created by this behaviour module","ref":"Vela.html#t:t/0","title":"Vela.t/0","type":"type"},{"doc":"Represents a value in the Vela structure","ref":"Vela.html#t:value/0","title":"Vela.value/0","type":"type"},{"doc":"The validator behaviour, implementations are used by Vela to validate new values to be inserted into the state.","ref":"Vela.Validator.html","title":"Vela.Validator","type":"behaviour"},{"doc":"When implemented, the module might be passed as validator to use Vela","ref":"Vela.Validator.html#c:valid?/2","title":"Vela.Validator.valid?/2","type":"callback"},{"doc":"The generic exception thrown when the access in invalid.Parameters:source the source of the exceptionfield the field the access to has failed","ref":"Vela.AccessError.html","title":"Vela.AccessError","type":"exception"},{"doc":"VelaThe tiny library to ease handling expiring invalidated cached series","ref":"readme.html","title":"Vela","type":"extras"},{"doc":"def deps do [ {:vela, &quot;~&gt; 0.6&quot;} ] end","ref":"readme.html#installation","title":"Vela - Installation","type":"extras"},{"doc":"0.6.3 — fix threshold to use compare_by/1 for cumbersome values0.6.1 — use threshold to prevent adding outliers to series0.6.0 — Vela.δ/1 / Vela.delta/1 returning a keyword [{serie, {min, max}}]","ref":"readme.html#changelog","title":"Vela - Changelog","type":"extras"},{"doc":"","ref":"readme.html#documentation","title":"Vela - Documentation","type":"extras"},{"doc":"Vela LifetimeVela is designed to collect and keep a set of series of different values. Each serie is supposed to keep validated, ordered, actual values. validation is being done upon insertion, using validator parameter for this particular serie (default: always allow;) if the validation fails, the value gets discarded immediatelyobsoleted values are removed by explicit call to purge/2sorting is done with sorter/compare_by pair of parameters; default sorting is none (LIFO) and the default comparator is itself.Consider the following example. Let’s declare Vela having two series, one with integers, another with dates. We want both to be sorted descending so that we could get the most actual values at any time.defmodule Checkers do def good_integer(:integers, int) when is_integer(int), do: true def good_integer(_, _), do: false def good_date(:dates, %Date{}), do: true def good_date(_, _), do: false def compare_dates(%Date{} = d1, %Date{} = d2), do: Date.compare(d1, d2) == :gt end defmodule Series do import Checkers use Vela, integers: [limit: 3, validator: &amp;good_integer/2, sorter: &amp;&gt;=/2], dates: [limit: 3, validator: &amp;good_date/2, sorter: &amp;compare_dates/2] endNow we can fill the struct with values.iex|1&gt; s = %Series{} #⇒ %Series{__errors__: [], dates: [], integers: []} iex|2&gt; put_in s, [:integers], 42 #⇒ %Series{__errors__: [], dates: [], integers: &#39;*&#39;} iex|3&gt; put_in s, [:integers], &quot;42&quot; #⇒ %Series{__errors__: [integers: &quot;42&quot;], dates: [], integers: []} iex|4&gt; put_in s, [:dates], Date.utc_today #⇒ %Series{__errors__: [], dates: [~D[2020-06-30]], integers: []} iex|5&gt; put_in s, [:dates], &quot;42&quot; #⇒ %Series{__errors__: [dates: &quot;42&quot;], dates: [], integers: []}Let’s see how sorting works.iex|6&gt; s ...|6&gt; |&gt; put_in([:integers], 10) ...|6&gt; |&gt; put_in([:integers], 20) ...|6&gt; |&gt; put_in([:integers], 30) ...|6&gt; |&gt; put_in([:integers], 40) ...|6&gt; |&gt; put_in([:integers], 0) %Series{__errors__: [], dates: [], integers: [40, 30, 20]}We instructed the serie to keep at most three values, sorted descending.The get to the most “relevant” values (heads of series,) one might use slice/1. It returns the keyword with all the actual values.iex|7&gt; Series.slice s #⇒ [integers: 40]To get to the head of each serie, one might use Access as shown below.iex|8&gt; get_in s, [:integers] #⇒ 40","ref":"lifetime.html","title":"Vela Lifetime","type":"extras"}]